@@ -8,14 +8,14 @@ class Server500(BaseException):
 class Post(object):
     def __init__(self, ruqqus, json):
         self.ruqqus = ruqqus
-        self.author = User(ruqqus, ruqqus.user(json['author']))
+        self.author = ruqqus.user(json['author'])
         self.body = json['body']
         self.body_html = json['body_html']
         self.created = datetime.datetime.utcfromtimestamp(json['created_utc'])
         self.domain = json['domain']
         self.edited = 0 if json['edited_utc'] == 0 else datetime.datetime.utcfromtimestamp(json['edited_utc'])
         self.embed_url = json['embed_url']
-        self.guild = Guild(ruqqus, ruqqus.guild(json['guild_name']))
+        self.guild = ruqqus.guild(json['guild_name'])
         self.id = json['id']
         self.is_archived = json['is_archived']
         self.is_banned = json['is_banned']
@@ -23,7 +23,7 @@ class Post(object):
         self.is_nsfl = json['is_nsfl']
         self.is_nsfw = json['is_nsfw']
         self.is_offensive = json['is_offensive']
-        self.original_guild = Guild(ruqqus, ruqqus.guild(json['original_guild_name']))
+        self.original_guild = ruqqus.guild(json['original_guild_name'])
         self.permalink = "https://ruqqus.com" + json['permalink']
         self.thumb_url = json['thumb_url']
         self.title = json['title']
@@ -34,7 +34,7 @@ class Post(object):
         if not v in [-1, 0, 1]: raise ValueError("Vote must be -1, 0, or 1")
         return self.ruqqus.vote_post(self.id, v)
     def reply(self, body):
-        return self.ruqqus.api_comment(self.id, self.id, body)
+        return self.ruqqus.api_comment(self.id, "t2"+self.id, body)
     def __str__(self):
         return self.title
     def __repr__(self):
@@ -126,11 +126,11 @@ class RuqqusAPI(object):
     
     # <official v1>
     def guild(self, boardname):
-        return self.session.get(self.base + "/guild/" + boardname).json()
+        return Guild(self, self.session.get(self.base + "/guild/" + boardname).json())
     def user(self, username):
-        return self.session.get(self.base + "/user/" + username).json()
+        return User(self, self.session.get(self.base + "/user/" + username).json())
     def post(self, pid):
-        return self.session.get(self.base + "/post/" + pid).json()
+        return Post(self, self.session.get(self.base + "/post/" + pid).json())
     def comment(self, cid):
         return self.session.get(self.base + "/comment/" + cid).json()
     # </official v1>
@@ -144,19 +144,39 @@ class RuqqusAPI(object):
         return r.status_code == 204
 
     def board_listing(self, name):
-        return self.handle(self.session.get(f"{self.base}/guild/{name}/listing"))
+        return [Post(self,x) for x in self.handle(self.session.get(f"{self.base}/guild/{name}/listing"))]
     
     def user_listing(self, username):
-        return self.handle(self.session.get(f"{self.base}/user/{username}/listing"))
+        return [Post(self, x) for x in self.handle(self.session.get(f"{self.base}/user/{username}/listing"))]
     
     def home(self, only=None, sort="hot", t="all", page=1):
-        return self.session.get(f"{self.base}/front/listing?sort={sort}&t={t}&page={page}" + ("" if not only else f"&only={only}")).json()
+        return [Post(self, x) for x in self.session.get(f"{self.base}/front/listing?sort={sort}&t={t}&page={page}" + ("" if not only else f"&only={only}")).json()]
     
     def front_all(self, page=1, sort="hot", t="all"):
-        return self.session.get(f"{self.base}/all/listing?page={page}&sort={sort}&t={t}").json()
+        return [Post(self, x) for x in self.session.get(f"{self.base}/all/listing?page={page}&sort={sort}&t={t}").json()]
     # </unofficial v1>
 
     # <unofficial>
+    def submit_post(self, title, board, url="", body="", file=None, over_18=False):
+        r = self.session.post("https://ruqqus.com/submit", data={
+            'title': title,
+            'board': board,
+            'url': url,
+            'body': body,
+            'over_18': over_18,
+        }, files={'file': file})
+        return r
+    
+    def delete_post(self, pid):
+        return self.session.post("https://ruqqus.com/delete_post/" + pid).status_code == 204
+
+    def create_guild(self, name, description):
+        r = self.session.post("https://ruqqus.com/create_guild", data={
+            'name': name,
+            'description': description,
+        })
+        return r.url if ('+' + name) in r.url else r
+
     def get_post_title(self, url):
         return self.session.get("https://ruqqus.com/api/submit/title?url=" + urllib.parse.quote_plus(url)).json()
 
@@ -215,10 +235,10 @@ class RuqqusAPI(object):
         return self.session.get("https://ruqqus.com/@%s/pic/profile" % username).url
     
     def vote_post(self, post_id, x="1"):
-        return self.session.post("https://ruqqus.com/api/vote/post/%s/%s" % (post_id, x)).status_code == 204
+        return True if (x:=self.session.post("https://ruqqus.com/api/vote/post/%s/%s" % (post_id, x))).status_code == 204 else x
     
     def vote_comment(self, comment_id, x="1"):
-        return self.session.post("https://ruqqus.com/api/vote/comment/%s/%s" % (comment_id, x)).status_code == 204
+        return True if (x:=self.session.post("https://ruqqus.com/api/vote/comment/%s/%s" % (comment_id, x))).status_code == 204 else x
     
     def admin_image_posts_listing(self, page=1):
         return self.session.get("https://ruqqus.com/admin/image_posts?page=" + str(page)).json()
